@@ -17,8 +17,8 @@ import copy
 import logging
 import socket
 
-from keystoneclient import adapter
-from keystoneclient import exceptions as ksc_exc
+from keystoneauth1 import adapter
+from keystoneauth1 import exceptions as ksa_exc
 from oslo_utils import importutils
 from oslo_utils import netutils
 import requests
@@ -140,7 +140,6 @@ class HTTPClient(_BaseHTTPClient):
         self.identity_headers = kwargs.get('identity_headers')
         self.auth_token = kwargs.get('token')
         self.language_header = kwargs.get('language_header')
-        self.last_request_id = None
         if self.identity_headers:
             if self.identity_headers.get('X-Auth-Token'):
                 self.auth_token = self.identity_headers.get('X-Auth-Token')
@@ -275,7 +274,6 @@ class HTTPClient(_BaseHTTPClient):
                        {'endpoint': endpoint, 'e': e})
             raise exc.CommunicationError(message=message)
 
-        self.last_request_id = resp.headers.get('x-openstack-request-id')
         resp, body_iter = self._handle_response(resp)
         self.log_http_response(resp)
         return resp, body_iter
@@ -315,7 +313,6 @@ class SessionClient(adapter.Adapter, _BaseHTTPClient):
     def __init__(self, session, **kwargs):
         kwargs.setdefault('user_agent', USER_AGENT)
         kwargs.setdefault('service_type', 'image')
-        self.last_request_id = None
         super(SessionClient, self).__init__(session, **kwargs)
 
     def request(self, url, method, **kwargs):
@@ -329,20 +326,19 @@ class SessionClient(adapter.Adapter, _BaseHTTPClient):
                                                       headers=headers,
                                                       data=data,
                                                       **kwargs)
-        except ksc_exc.RequestTimeout as e:
+        except ksa_exc.ConnectTimeout as e:
             conn_url = self.get_endpoint(auth=kwargs.get('auth'))
             conn_url = "%s/%s" % (conn_url.rstrip('/'), url.lstrip('/'))
             message = ("Error communicating with %(url)s %(e)s" %
                        dict(url=conn_url, e=e))
             raise exc.InvalidEndpoint(message=message)
-        except ksc_exc.ConnectionRefused as e:
+        except ksa_exc.ConnectFailure as e:
             conn_url = self.get_endpoint(auth=kwargs.get('auth'))
             conn_url = "%s/%s" % (conn_url.rstrip('/'), url.lstrip('/'))
             message = ("Error finding address for %(url)s: %(e)s" %
                        dict(url=conn_url, e=e))
             raise exc.CommunicationError(message=message)
 
-        self.last_request_id = resp.headers.get('x-openstack-request-id')
         return self._handle_response(resp)
 
 
